@@ -26,6 +26,7 @@
 #include <sys/types.h>
 
 #include "calctool.h"
+#include "unittest.h"
 #include "get.h"
 #include "display.h"
 #include "functions.h"
@@ -619,7 +620,7 @@ usage(char *progname)
      * the second is the program version number.
      */
     FPRINTF(stderr, _("%s version %s\n\n"), progname, VERSION);
-    FPRINTF(stderr, _("Usage: %s: [-D] [-E] [-a accuracy] "), progname);
+    FPRINTF(stderr, _("Usage: %s: [-D] [-E] [-u] [-a accuracy] "), progname);
     FPRINTF(stderr, _("\t\t [-?] [-v] [-h]\n"));
     exit(1);
 }
@@ -655,6 +656,10 @@ get_options(int argc, char *argv[])      /* Extract command line options. */
                     }
                     break;
 
+                case 'u':
+                    unittest();
+                    break;
+
                 case '?' :
                 case 'v' : 
                 case 'h' :                 
@@ -743,15 +748,15 @@ main(int argc, char **argv)
         v->appname = strdup(argv[0]);
     }
     
-    get_options(argc, argv);   /* Get command line arguments. */
+    init_state();
     
+    get_options(argc, argv);   /* Get command line arguments. */
+    ui_init(&argc, &argv);     /* Initialise UI */
+    resources_init();          /* Initialise configuration */
+
     v->radix = get_radix();    /* Locale specific radix string. */
     v->tsep  = get_tsep();     /* Locale specific thousands separator. */
     v->tsep_count = get_tsep_count();
-    
-    init_state();
-    ui_init(&argc, &argv);     /* Initialise UI */
-    resources_init();          /* Initialise configuration */
 
     init_text();               /* Setup text strings depending upon language. */
     read_resources();          /* Read resources from merged database. */
