@@ -46,60 +46,74 @@ static char *make_eng_sci(int *, int);
  */
 
 void
-localize_number(char *dest, const char *src, int dest_length)
+localize_expression(char *dest, const char *src, int dest_length)
 {
-    char tnum[MAX_LOCALIZED], *dstp;
+    GString *clean, *output;
+    const char *c, *d;
+    int digit_count = -1;
+    gboolean after_radix = FALSE;
 
-    if (!v->error && v->show_tsep && v->base == DEC) {
-        const char *radixp, *srcp;
-        int n, i;
-        size_t tsep_len;
-
-        /* Process the fractional part (if any). */
-        srcp = src + strlen(src) - 1;
-        dstp = tnum;
-        if ((radixp = strchr(src, '.')) != NULL) {
-            while (srcp != radixp) {
-                *dstp++ = *srcp--;
-            }
-            *dstp++ = *srcp--;
-        }
-
-        /* Process the integer part, add in thousand separators. */
-        tsep_len = strlen(v->tsep);
-        n = 0;
-        while (srcp >= src) {
-            *dstp++ = *srcp--;
-            n++;
-            if (n == 3 && srcp >= src && *srcp != '-') {
-                for (i = tsep_len - 1; i >= 0; i--) {
-                    *dstp++ = v->tsep[i];
-                }
-                n = 0;
-            }
-        }
-        *dstp++ = '\0';
-
-        /* Move from scratch pad to fnum, reversing the character order. */
-        srcp = tnum + strlen(tnum) - 1;
-        dstp = dest;
-        while (srcp >= tnum) {
-            *dstp++ = *srcp--;
-        }
-        *dstp++ = '\0';
-    } else {
+    /* Only modify if valid */
+    if (v->error || v->base != DEC) {
         STRNCPY(dest, src, dest_length - 1);
+        return;
     }
-    dstp = strchr(dest, '.');
-    if (dstp != NULL) {
-        size_t radix_len;
 
-        radix_len = strlen(v->radix);
-        if (radix_len != 1) {
-            memmove(dstp + radix_len, dstp + 1, strlen (dstp + 1) + 1);
+    /* Remove separators if not supported */
+    clean = g_string_sized_new(strlen(src));
+    for (c = src; *c; c++) {
+        if (strncmp(c, v->tsep, strlen(v->tsep)) == 0) {
+            c += strlen(v->tsep) - 1;
+        }
+        else {
+            g_string_append_c(clean, *c);
         }
-        MEMCPY(dstp, v->radix, radix_len);
     }
+
+    if (!v->show_tsep) {
+        STRNCPY(dest, clean->str, dest_length - 1);
+        g_string_free(clean, TRUE);
+        return;
+    }
+
+    /* Scan expression looking for numbers and inserting separators */
+    output = g_string_sized_new(dest_length);
+    for (c = clean->str; *c; c++) {
+        /* Insert separators between digits */
+        if (*c >= '0' && *c <= '9') {
+            /* Read ahead to find the number of digits */
+            if (digit_count < 0) {
+                digit_count = 1;
+                for (d = c + 1; *d >= '0' && *d <= '9'; d++)
+                    digit_count++;
+            }
+            
+            g_string_append_c(output, *c);
+            
+            /* Insert separator after nth digit */
+            if (!after_radix && digit_count > 1 && digit_count % v->tsep_count == 1) {
+                g_string_append(output, v->tsep);
+            }
+            digit_count--;
+        }
+        /* Ignore digits after the radix */
+        else if (strncmp(c, v->radix, strlen(v->radix)) == 0) {
+            digit_count = -1;
+            after_radix = TRUE;
+            c += strlen(v->radix) - 1;
+            g_string_append(output, v->radix);
+        }
+        /* Reset when encountering other characters (e.g. '+') */
+        else {
+            digit_count = -1;
+            after_radix = FALSE;
+            g_string_append_c(output, *c);
+        }
+    }
+    
+    STRNCPY(dest, output->str, dest_length - 1);
+    g_string_free(output, TRUE);
+    g_string_free(clean, TRUE);    
 }
 
 
@@ -121,18 +135,14 @@ char_val(char chr)
 void
 clear_display(int initialise)
 {
-    int i;
-    
     v->pointed = 0;
     v->toclear = 1;
-    i = 0;
-    mpcim(&i, v->MPdisp_val);
+    do_zero(v->MPdisp_val);
     STRNCPY(v->display, make_number(v->MPdisp_val, v->base, FALSE), 
             MAXLINE - 1);
     ui_set_display(v->display, -1);
 
     if (initialise == TRUE) {
-        v->show_paren = 0;
         v->numsptr    = 0;
         v->noparens   = 0;
         ui_set_hyperbolic_state(FALSE);          /* Also clears v->hyperbolic. */
@@ -145,15 +155,12 @@ clear_display(int initialise)
 void
 initialise()
 {
-    int i;
- 
     v->error         = 0;           /* Currently no display error. */
     v->cur_op        = -1;         /* No arithmetic operator defined yet. */
     v->old_cal_value = -1;
-    i = 0;
-    mpcim(&i, v->MPresult);         /* No previous result yet. */
-    mpcim(&i, v->MPdisp_val);         
-    mpcim(&i, v->MPlast_input);
+    do_zero(v->MPresult);         /* No previous result yet. */
+    do_zero(v->MPdisp_val);         
+    do_zero(v->MPlast_input);
   
     v->new_input = 1;               /* Value zero is on calculator display */
 
@@ -172,12 +179,11 @@ make_fixed(int *MPnumber, char *str, int base, int cmax, int toclear)
     int MP1base[MP_SIZE], MP1[MP_SIZE], MP2[MP_SIZE], MPval[MP_SIZE];
     int ndig;                   /* Total number of digits to generate. */
     int ddig;                   /* Number of digits to left of decimal sep. */
-    int dval, n;
+    int dval, n, i;
  
     optr = str;
     mpabs(MPnumber, MPval);
-    n = 0;
-    mpcim(&n, MP1);
+    do_zero(MP1);
     if (mplt(MPnumber, MP1)) {
         *optr++ = '-';
     }
@@ -207,7 +213,8 @@ make_fixed(int *MPnumber, char *str, int base, int cmax, int toclear)
 
     while (ndig-- > 0) {
         if (ddig-- == 0) {
-            *optr++ = '.';
+            for (i = 0; i < strlen(v->radix); i++)
+                *optr++ = v->radix[i];
         }
         mpmul(MPval, MP1base, MPval);
         mpcmi(MPval, &dval);
@@ -226,15 +233,16 @@ make_fixed(int *MPnumber, char *str, int base, int cmax, int toclear)
     }
     v->pointed = 0;
 
-    if (!v->show_zeroes && v->accuracy != 0) {
-        optr = str + strlen(str) - 1;
-        while (*optr == '0') {
-            optr--;
+    /* Strip off trailing zeroes */
+    if (!v->show_zeroes) {
+        for (i = strlen(str) - 1; i > 1 && str[i] == '0'; i--) {
+            str[i] = '\0';
+        }
+        
+        /* If no fractional part discard radix */
+        if (strlen(str) >= strlen(v->radix) && strcmp(str + strlen(str) - strlen(v->radix), v->radix) == 0) {
+            str[strlen(str) - strlen(v->radix)] = '\0';
         }
-    if (optr < str || *optr != '.') {
-        optr++;
-    }
-        *optr = '\0';
     }
 
     return(str);
@@ -290,8 +298,7 @@ make_eng_sci(int *MPnumber, int base)
     }
     optr = v->snum;
     mpabs(MPnumber, MPval);
-    n = 0;
-    mpcim(&n, MP1);
+    do_zero(MP1);
     if (mplt(MPnumber, MP1)) {
         *optr++ = '-';
     }
@@ -308,8 +315,7 @@ make_eng_sci(int *MPnumber, int base)
     mpcim(&n, MP1);
     mpdiv(MP1, MP10base, MPatmp);
 
-    n = 0;
-    mpcim(&n, MP1);
+    do_zero(MP1);
     if (!mpeq(MPmant, MP1)) {
         while (!eng && mpge(MPmant, MP10base)) {
             exp += 10;
@@ -393,8 +399,7 @@ MPstr_to_num(char *str, enum base_type base, int *MPval)
     char *lnp = ui_get_localized_numeric_point();
     assert(lnp);
 
-    i = 0;
-    mpcim(&i, MPval);
+    do_zero(MPval);
     mpcim(&basevals[(int) base], MPbase);
 
     optr = str;
@@ -458,7 +463,7 @@ MPstr_to_num(char *str, enum base_type base, int *MPval)
 void
 paren_disp(int key)
 {
-    int i, n;
+    int n;
     char *text;
 
 /*  If the character is a Delete, clear the whole line, and exit parenthesis
@@ -478,8 +483,7 @@ paren_disp(int key)
     case KEY_CLEAR:
         v->noparens = v->numsptr = 0;
         v->cur_op = -1;
-        i = 0;
-        mpcim(&i, v->MPdisp_val);
+        do_zero(v->MPdisp_val);
         show_display(v->MPdisp_val);
         return;
     case KEY_BACKSPACE:
@@ -525,9 +529,7 @@ paren_disp(int key)
     }
 
     n = (n < MAX_DIGITS) ? 0 : n - MAX_DIGITS;
-    v->show_paren = 1;       /* Hack to get ui_set_display to really display it. */
     ui_set_display(&v->display[n], -1);
-    v->show_paren = 0;
 }
 
 
@@ -588,7 +590,7 @@ refresh_display(int cursor)
             }
             ans = make_number(e->ans, v->base, TRUE);
 
-            localize_number(localized, ans, MAX_LOCALIZED);
+            localize_expression(localized, ans, MAX_LOCALIZED);
             t = str_replace(str, "Ans", localized);
             free(str);
             str = t;
