@@ -205,7 +205,7 @@ peek_previous_state(void)
 }
 
 static int
-is_undo_step()
+is_undo_step(void)
 {
     return(v->h.current != v->h.begin);
 }
@@ -237,7 +237,7 @@ do_accuracy(int value)     /* Set display accuracy. */
 
 
 void
-do_business()     /* Perform special business mode calculations. */
+do_business(void)     /* Perform special business mode calculations. */
 {
     if (v->current == KEY_FINC_CTRM) {
         calc_ctrm(v->MPdisp_val);
@@ -282,9 +282,9 @@ exp_insert(char *text, int cursor)
 
 
 void
-exp_clear()
+exp_clear(void)
 {
-    exp_replace("Ans");
+    exp_replace("");
 }
 
 
@@ -292,7 +292,11 @@ int
 usable_num(int MPnum[MP_SIZE])
 {
     struct exprm_state *e = get_state();
-    return ce_parse(e->expression, MPnum);
+    if (e->expression[0] == '\0') {
+        return ce_parse("0", MPnum);
+    } else {
+        return ce_parse(e->expression, MPnum);
+    }
 }
 
 
@@ -333,7 +337,10 @@ exp_backspace(int cursor)
     if (cursor < 0) {
         if (exp_has_postfix(e->expression, "Ans")) {
             char *ans = make_number(e->ans, v->base, FALSE);
-            str_replace(&e->expression, "Ans", ans);
+            char *t;
+            t = str_replace(e->expression, "Ans", ans);
+            free(e->expression);
+            e->expression = t;
         } else {
             for (i = 0; i < 10; i++) {
                 SNPRINTF(buf, MAXLINE, "R%d", i);
@@ -389,7 +396,7 @@ exp_replace(char *text)
 
 
 static void
-exp_negate()
+exp_negate(void)
 {
     struct exprm_state *e = get_state();
 
@@ -407,7 +414,7 @@ exp_negate()
 
 
 static void
-exp_inv()
+exp_inv(void)
 {
     struct exprm_state *e = get_state();
 
@@ -424,44 +431,30 @@ exp_inv()
 }
 
 
-void
-str_replace(char **str, char *from, char *to)
+char *
+str_replace(char *str, char *from, char *to)
 {
-    int i, flen, len;
-
-    assert(str);
-    assert(from);
-    assert(to);
-
-    if (!*str) {
-        return;
-    }
-
-    i = 0;
-    len = strlen(*str);
-    flen = strlen(from);
-
-    for (i = 0; len-i >= flen; i++) {
-        if (!strncasecmp(from, *str+i, flen)) {
-            char *print;
-            int j = i+flen;
-            char *prefix = malloc(i+1);
-            char *postfix = malloc(len-j+1);
-
-            assert(prefix && postfix);
-            memset(prefix, 0, i+1);
-            memset(postfix, 0, len-j+1);
-            MEMCPY(prefix, *str, i);
-            MEMCPY(postfix, *str+i+flen, len-j);
-
-            print = malloc(strlen(to)+i+len-j+1);
-            SPRINTF(print, "%s%s%s", prefix, to, postfix);
-            free(prefix);
-            free(postfix);
-            free(*str);
-            *str = print;
+    char output[MAXLINE];
+    int offset = 0;
+    char *c;
+    int flen = strlen(from);
+    int tlen = strlen(to);
+    
+    for (c = str; *c && offset < MAXLINE - 1; c++, offset++) {
+        if (strncasecmp(from, c, flen) == 0) {
+            SNPRINTF(output + offset, MAXLINE - offset, to);
+            c += flen - 1;
+            offset += tlen - 1;
+        } else {
+            output[offset] = *c;
         }
     }
+
+    if (offset >= MAXLINE)
+        offset = MAXLINE - 1;
+    output[offset] = '\0';
+    
+    return strdup(output);
 }
 
 
@@ -564,9 +557,26 @@ do_expression(int function, int arg, int cursor)
             /* Solve the equation */
             } else {
                 int MPval[MP_SIZE];
-                char *message = NULL;
+                char *c, *message = NULL;
+                GString *clean;
+                int result;
                 
-                switch (ce_parse(e->expression, MPval)) {
+                /* Remove thousands separators and use english radix */
+                clean = g_string_sized_new(strlen(e->expression));
+                for (c = e->expression; *c; c++) {
+                    if (strncmp(c, v->tsep, strlen(v->tsep)) == 0) {
+                        c += strlen(v->tsep) - 1;
+                    } else if (strncmp(c, v->radix, strlen(v->radix)) == 0) {
+                        g_string_append_c(clean, '.');
+                        c += strlen(v->radix) - 1;
+                    } else {
+                        g_string_append_c(clean, *c);
+                    }
+                }
+                result = ce_parse(clean->str, MPval);
+                g_string_free(clean, TRUE);
+                
+                switch (result) {
                     case 0:
                         mpstr(MPval, e->ans);
                         exp_replace("Ans");
@@ -615,6 +625,11 @@ do_expression(int function, int arg, int cursor)
                          buttons[function].symname,
                          e->expression);
                 exp_replace(buf);
+            } else if ((((buttons[function].id == KEY_ADD) 
+                         || (buttons[function].id == KEY_SUBTRACT))
+                        && !strcmp(e->expression, "Ans")
+                        && (ex && ex->clear))) {
+                exp_replace(buttons[function].symname);
             } else {
                 if (buttons[function].flags & FUNC) {
                     SNPRINTF(buf, MAXLINE, "%s(", buttons[function].symname);
@@ -634,7 +649,7 @@ do_expression(int function, int arg, int cursor)
 
 
 void
-do_calc()      /* Perform arithmetic calculation and display result. */
+do_calc(void)      /* Perform arithmetic calculation and display result. */
 {
     double dval, dres;
     int MP1[MP_SIZE], MP2[MP_SIZE];
@@ -695,6 +710,12 @@ do_calc()      /* Perform arithmetic calculation and display result. */
                 mpcmim(MP1, MP1);
                 mpmul(MP1, v->MPdisp_val, MP2);
                 mpsub(v->MPresult, MP2, v->MPresult);
+
+                do_zero(MP1);
+                if ((mplt(v->MPdisp_val, MP1) && mpgt(v->MPresult, MP1)) ||
+                    mplt(v->MPresult, MP1)) { 
+                    mpadd(v->MPresult, v->MPdisp_val, v->MPresult);
+                }
             }
             break;
 
@@ -868,7 +889,7 @@ do_percent(void)
 
 /* Clear the calculator display and re-initialise. */
 void
-do_clear()
+do_clear(void)
 {
     clear_display(TRUE);
     if (v->error) {
@@ -880,7 +901,7 @@ do_clear()
 
 /* Clear the calculator display. */
 void
-do_clear_entry()
+do_clear_entry(void)
 {
     clear_display(FALSE);
 }
@@ -945,7 +966,7 @@ do_constant(int index)
 
 /* Remove the last numeric character typed. */
 void
-do_backspace()
+do_backspace(void)
 {
     size_t len;
 
@@ -973,14 +994,14 @@ do_backspace()
     MPstr_to_num(v->display, v->base, v->MPdisp_val);
 
     if (v->dtype == FIX) {
-        STRCPY(v->fnum, v->display);
+        STRNCPY(v->fnum, v->display, MAX_LOCALIZED - 1);
         ui_set_display(v->fnum, -1);
     }
 }
 
 
 void
-do_delete()
+do_delete(void)
 {
     /* Not required in ltr mode */
 }
@@ -1029,11 +1050,11 @@ do_exchange(int index)
 
 /* Get exponential number. */
 void
-do_expno()
+do_expno(void)
 {
     v->pointed = (strchr(v->display, '.') != NULL);
     if (!v->new_input) {
-        STRCPY(v->display, "1.0 +");
+        STRNCPY(v->display, "1.0 +", MAX_LOCALIZED - 1);
         v->new_input = v->pointed = 1;
     } else if (!v->pointed) {
         STRNCAT(v->display, ". +", 3);
@@ -1068,8 +1089,7 @@ do_factorial(int *MPval, int *MPres)
 
     mpstr(MPval, MPa);
     mpcmim(MPval, MP1);
-    i = 0;
-    mpcim(&i, MP2);
+    do_zero(MP2);
     if (mpeq(MPval, MP1) && mpge(MPval, MP2)) {   /* Only positive integers. */
         i = 1;
         if (mpeq(MP1, MP2)) {                     /* Special case for 0! */
@@ -1215,7 +1235,7 @@ do_immedfunc(int s[MP_SIZE], int t[MP_SIZE])
 
 
 void
-do_immed()
+do_immed(void)
 {
     do_immedfunc(v->MPdisp_val, v->MPdisp_val);
     show_display(v->MPdisp_val);
@@ -1223,7 +1243,7 @@ do_immed()
 
 
 void
-do_number()
+do_number(void)
 {
     int offset;
 
@@ -1280,7 +1300,7 @@ do_numtype(enum num_type n)   /* Set number display type. */
 
 
 void
-do_paren()
+do_paren(void)
 {
     ui_set_statusbar("", "");
 
@@ -1391,7 +1411,7 @@ do_rcl_reg(int reg, int value[MP_SIZE])
 
 
 void
-syntaxdep_show_display()
+syntaxdep_show_display(void)
 {
     switch (v->syntax) {
         case NPA:
@@ -1409,11 +1429,11 @@ syntaxdep_show_display()
 
 
 void
-do_point()                   /* Handle numeric point. */
+do_point(void)                   /* Handle numeric point. */
 {
     if (!v->pointed) {
         if (v->toclear) {
-            STRCPY(v->display, ".");
+            STRNCPY(v->display, ".", MAX_LOCALIZED - 1);
             v->toclear = 0;
         } else {
             STRCAT(v->display, ".");
@@ -1450,7 +1470,7 @@ do_portionfunc(int num[MP_SIZE])
 
 
 void
-do_portion()
+do_portion(void)
 {
     do_portionfunc(v->MPdisp_val);
     show_display(v->MPdisp_val);
