@@ -215,20 +215,20 @@ static struct button_widget button_widgets[] = {
     { GDK_parenright, GDK_parenright, 0 }},
 
     {KEY_ADD,                "add",
-    { GDK_SHIFT_MASK, 0,        0,          0 },
-    { GDK_plus,       GDK_plus, GDK_KP_Add, 0 }},
+    { 0,        GDK_SHIFT_MASK, 0,          0 },
+    { GDK_plus, GDK_plus,       GDK_KP_Add, 0 }},
 
     {KEY_SUBTRACT,           "subtract",
-    { 0,         0,               0,      0 },
-    { GDK_minus, GDK_KP_Subtract, GDK_R4, 0 }},
+    { 0,         GDK_SHIFT_MASK, 0,               0,      0 },
+    { GDK_minus, GDK_minus,      GDK_KP_Subtract, GDK_R4, 0 }},
 
     {KEY_MULTIPLY,           "multiply",
-    { 0,            GDK_SHIFT_MASK, 0,               0,      0 },
-    { GDK_asterisk, GDK_asterisk,   GDK_KP_Multiply, GDK_R6, 0 }},
+    { 0,            GDK_SHIFT_MASK, 0,            GDK_SHIFT_MASK, 0,               0,      0 },
+    { GDK_asterisk, GDK_asterisk,   GDK_multiply, GDK_multiply,   GDK_KP_Multiply, GDK_R6, 0 }},
 
     {KEY_DIVIDE,             "divide",
-    { 0,         GDK_SHIFT_MASK, 0,             0,      GDK_SHIFT_MASK, 0 },
-    { GDK_slash, GDK_slash,      GDK_KP_Divide, GDK_R5, GDK_slash,      0 }},
+    { 0,         GDK_SHIFT_MASK, 0,            GDK_SHIFT_MASK, 0,             0,      GDK_SHIFT_MASK, 0 },
+    { GDK_slash, GDK_slash,      GDK_division, GDK_division,   GDK_KP_Divide, GDK_R5, GDK_slash,      0 }},
 
     {KEY_CHANGE_SIGN,        "change_sign_simple",
     { GDK_SHIFT_MASK, 0 },
@@ -543,15 +543,15 @@ ui_set_accuracy(int accuracy)
     widget = gtk_bin_get_child(GTK_BIN(GET_WIDGET("acc_item_other")));
     gtk_label_set_markup_with_mnemonic(GTK_LABEL(widget), text);
 
-    desc = g_strdup_printf(ngettext("Set accuracy from 0 to %d numeric places.",
+    desc = g_strdup_printf(ngettext("Set accuracy from 0 to %d numeric place.",
                                     "Set accuracy from 0 to %d numeric places.",
                                     MAXACC),
                            MAXACC);
 
-    /* Translator: This refers to the current accuracy setting */
-    current = g_strdup_printf(ngettext("Currently set to %d places.",
+    /* Translators: This refers to the current accuracy setting */
+    current = g_strdup_printf(ngettext("Currently set to %d place.",
                                        "Currently set to %d places.",
-                                       accuracy),
+                                       accuracy == 1),
                               accuracy);
     tooltip = g_strdup_printf ("%s %s [A]", desc, current);
     gtk_widget_set_tooltip_text (GET_WIDGET("calc_accuracy_button"), tooltip);
@@ -943,27 +943,6 @@ set_bit_panel(void)
     }
 }
 
-static gboolean
-redo_display(gpointer data)
-{
-    gchar *text;
-    GtkTextIter start, end, cursor;
-    gint cursor_position;
-
-    gtk_text_buffer_get_start_iter(X->display_buffer, &start);
-    gtk_text_buffer_get_end_iter(X->display_buffer, &end);
-    text = gtk_text_buffer_get_text(X->display_buffer, &start, &end, FALSE);
-    
-    g_object_get(G_OBJECT(X->display_buffer), "cursor-position", &cursor_position, NULL);
-
-    gtk_text_buffer_set_text(X->display_buffer, text, -1);    
-    gtk_text_buffer_get_iter_at_offset(X->display_buffer, &cursor, cursor_position);
-    gtk_text_buffer_place_cursor(X->display_buffer, &cursor);
-
-    g_free(text);
-    
-    return FALSE;
-}
 
 void
 ui_set_display(char *str, int cursor)
@@ -975,8 +954,8 @@ ui_set_display(char *str, int cursor)
     if (str == NULL || str[0] == '\0') {
         str = " ";
     } else {
-        if (v->syntax == EXPRS || (v->syntax == NPA && v->noparens == 0)) {
-            localize_expression(localized, str, MAX_LOCALIZED, &cursor);
+        if (v->noparens == 0) {
+            localize_expression(localized, str, MAX_LOCALIZED);
             str = localized;
         }
     }
@@ -990,13 +969,6 @@ ui_set_display(char *str, int cursor)
     }
     gtk_text_buffer_place_cursor(X->display_buffer, &iter);
     
-    /* This is a workaround for bug #524602.
-     * Basically the above code can cause the display to disappear when going from
-     * a display that is wider than the widget to one that is thinner. The following
-     * causes the display to be set twice which seems to work the second time.
-     */
-    g_idle_add(redo_display, NULL);
-    
     /* Align to the right */
     if (cursor < 0) {
         adj = gtk_scrolled_window_get_hadjustment(
@@ -1070,10 +1042,14 @@ ui_beep()
 }
 
 
-const char *
+char *
 ui_get_localized_numeric_point(void)
 {
-    return strdup(v->radix);
+    const char *decimal_point;
+
+    decimal_point = localeconv()->decimal_point;
+
+    return (g_locale_to_utf8(decimal_point, -1, NULL, NULL, NULL));
 }
 
 
